@@ -1,7 +1,7 @@
 /* Definitions for GNU multiple precision functions.   -*- mode: c -*-
 
-Copyright 1991, 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2001, 2002
-Free Software Foundation, Inc.
+Copyright 1991, 1993, 1994, 1995, 1996, 1997, 1999, 2000, 2001, 2002, 2003,
+2004 Free Software Foundation, Inc.
 
 This file is part of the GNU MP Library.
 
@@ -28,7 +28,7 @@ MA 02111-1307, USA. */
 
 
 /* Instantiated by configure. */
-#if ! __GMP_WITHIN_CONFIGURE
+#if ! defined (__GMP_WITHIN_CONFIGURE)
 #define __GMP_BITS_PER_MP_LIMB             32
 #define __GMP_HAVE_HOST_CPU_FAMILY_power   0
 #define __GMP_HAVE_HOST_CPU_FAMILY_powerpc 0
@@ -36,7 +36,7 @@ MA 02111-1307, USA. */
 #define GMP_NAIL_BITS                      0
 #endif
 #define GMP_NUMB_BITS     (GMP_LIMB_BITS - GMP_NAIL_BITS)
-#define GMP_NUMB_MASK     ((~(mp_limb_t) 0) >> GMP_NAIL_BITS)
+#define GMP_NUMB_MASK     ((~ __GMP_CAST (mp_limb_t, 0)) >> GMP_NAIL_BITS)
 #define GMP_NUMB_MAX      GMP_NUMB_MASK
 #define GMP_NAIL_MASK     (~ GMP_NUMB_MASK)
 
@@ -56,7 +56,7 @@ MA 02111-1307, USA. */
 #undef __need_size_t
 
 /* Instantiated by configure. */
-#if ! __GMP_WITHIN_CONFIGURE
+#if ! defined (__GMP_WITHIN_CONFIGURE)
 /* #undef _LONG_LONG_LIMB */
 #define __GMP_LIBGMP_DLL  1
 #endif
@@ -164,7 +164,7 @@ MA 02111-1307, USA. */
 #endif
 
 
-#ifdef _SHORT_LIMB
+#ifdef __GMP_SHORT_LIMB
 typedef unsigned int		mp_limb_t;
 typedef int			mp_limb_signed_t;
 #else
@@ -305,7 +305,8 @@ typedef __mpq_struct *mpq_ptr;
   || defined (_STDIO_INCLUDED)        /* HPUX */                \
   || defined (__dj_include_stdio_h_)  /* DJGPP */               \
   || defined (_FILE_DEFINED)          /* Microsoft */          \
-  || defined (__STDIO__)              /* Apple MPW MrC */
+  || defined (__STDIO__)              /* Apple MPW MrC */       \
+  || defined (_MSL_STDIO_H)           /* Metrowerks */
 #define _GMP_H_HAVE_FILE 1
 #endif
 
@@ -350,6 +351,20 @@ typedef __mpq_struct *mpq_ptr;
 #endif
 
 
+/* __GMP_CAST allows us to use static_cast in C++, so our macros are clean
+   to "g++ -Wold-style-cast".
+
+   Casts in "extern inline" code within an extern "C" block don't induce
+   these warnings, so __GMP_CAST only needs to be used on documented
+   macros.  */
+
+#ifdef __cplusplus
+#define __GMP_CAST(type, expr)  (static_cast<type> (expr))
+#else
+#define __GMP_CAST(type, expr)  ((type) (expr))
+#endif
+
+
 /* An empty "throw ()" means the function doesn't throw any C++ exceptions,
    this can save some stack frame info in applications.
 
@@ -393,10 +408,12 @@ typedef __mpq_struct *mpq_ptr;
    "extern" is apparently ignored, so foo is inlined if possible but also
    emitted as a global, which causes multiple definition errors when
    building a shared libgmp.  */
+#ifdef __SCO_VERSION__
 #if __SCO_VERSION__ > 400000000 && __STDC__ != 1 \
   && ! defined (__GMP_EXTERN_INLINE)
 #define __GMP_EXTERN_INLINE  static inline
 #endif
+#endif
 
 /* C++ always has "inline" and since it's a normal feature the linker should
    discard duplicate non-inlined copies, or if it doesn't then that's a
@@ -503,17 +520,17 @@ __GMP_DECLSPEC void gmp_randclear __GMP_PROTO ((gmp_randstate_t));
 __GMP_DECLSPEC int gmp_asprintf __GMP_PROTO ((char **, const char *, ...));
 
 #define gmp_fprintf __gmp_fprintf
-#if _GMP_H_HAVE_FILE
+#ifdef _GMP_H_HAVE_FILE
 __GMP_DECLSPEC int gmp_fprintf __GMP_PROTO ((FILE *, const char *, ...));
 #endif
 
 #define gmp_obstack_printf __gmp_obstack_printf
-#if _GMP_H_HAVE_OBSTACK
+#if defined (_GMP_H_HAVE_OBSTACK)
 __GMP_DECLSPEC int gmp_obstack_printf __GMP_PROTO ((struct obstack *, const char *, ...));
 #endif
 
 #define gmp_obstack_vprintf __gmp_obstack_vprintf
-#if _GMP_H_HAVE_OBSTACK && _GMP_H_HAVE_VA_LIST
+#if defined (_GMP_H_HAVE_OBSTACK) && defined (_GMP_H_HAVE_VA_LIST)
 __GMP_DECLSPEC int gmp_obstack_vprintf __GMP_PROTO ((struct obstack *, const char *, va_list));
 #endif
 
@@ -527,27 +544,27 @@ __GMP_DECLSPEC int gmp_snprintf __GMP_PROTO ((char *, size_t, const char *, ...)
 __GMP_DECLSPEC int gmp_sprintf __GMP_PROTO ((char *, const char *, ...));
 
 #define gmp_vasprintf __gmp_vasprintf
-#if _GMP_H_HAVE_VA_LIST
+#if defined (_GMP_H_HAVE_VA_LIST)
 __GMP_DECLSPEC int gmp_vasprintf __GMP_PROTO ((char **, const char *, va_list));
 #endif
 
 #define gmp_vfprintf __gmp_vfprintf
-#if _GMP_H_HAVE_FILE && _GMP_H_HAVE_VA_LIST
+#if defined (_GMP_H_HAVE_FILE) && defined (_GMP_H_HAVE_VA_LIST)
 __GMP_DECLSPEC int gmp_vfprintf __GMP_PROTO ((FILE *, const char *, va_list));
 #endif
 
 #define gmp_vprintf __gmp_vprintf
-#if _GMP_H_HAVE_VA_LIST
+#if defined (_GMP_H_HAVE_VA_LIST)
 __GMP_DECLSPEC int gmp_vprintf __GMP_PROTO ((const char *, va_list));
 #endif
 
 #define gmp_vsnprintf __gmp_vsnprintf
-#if _GMP_H_HAVE_VA_LIST
+#if defined (_GMP_H_HAVE_VA_LIST)
 __GMP_DECLSPEC int gmp_vsnprintf __GMP_PROTO ((char *, size_t, const char *, va_list));
 #endif
 
 #define gmp_vsprintf __gmp_vsprintf
-#if _GMP_H_HAVE_VA_LIST
+#if defined (_GMP_H_HAVE_VA_LIST)
 __GMP_DECLSPEC int gmp_vsprintf __GMP_PROTO ((char *, const char *, va_list));
 #endif
 
@@ -555,7 +572,7 @@ __GMP_DECLSPEC int gmp_vsprintf __GMP_PROTO ((char *, const char *, va_list));
 /**************** Formatted input routines.  ****************/
 
 #define gmp_fscanf __gmp_fscanf
-#if _GMP_H_HAVE_FILE
+#ifdef _GMP_H_HAVE_FILE
 __GMP_DECLSPEC int gmp_fscanf __GMP_PROTO ((FILE *, const char *, ...));
 #endif
 
@@ -566,17 +583,17 @@ __GMP_DECLSPEC int gmp_scanf __GMP_PROTO ((const char *, ...));
 __GMP_DECLSPEC int gmp_sscanf __GMP_PROTO ((const char *, const char *, ...));
 
 #define gmp_vfscanf __gmp_vfscanf
-#if _GMP_H_HAVE_FILE && _GMP_H_HAVE_VA_LIST
+#if defined (_GMP_H_HAVE_FILE) && defined (_GMP_H_HAVE_VA_LIST)
 __GMP_DECLSPEC int gmp_vfscanf __GMP_PROTO ((FILE *, const char *, va_list));
 #endif
 
 #define gmp_vscanf __gmp_vscanf
-#if _GMP_H_HAVE_VA_LIST
+#if defined (_GMP_H_HAVE_VA_LIST)
 __GMP_DECLSPEC int gmp_vscanf __GMP_PROTO ((const char *, va_list));
 #endif
 
 #define gmp_vsscanf __gmp_vsscanf
-#if _GMP_H_HAVE_VA_LIST
+#if defined (_GMP_H_HAVE_VA_LIST)
 __GMP_DECLSPEC int gmp_vsscanf __GMP_PROTO ((const char *, const char *, va_list));
 #endif
 
@@ -588,7 +605,7 @@ __GMP_DECLSPEC int gmp_vsscanf __GMP_PROTO ((const char *, const char *, va_list
 __GMP_DECLSPEC void *_mpz_realloc __GMP_PROTO ((mpz_ptr, mp_size_t));
 
 #define mpz_abs __gmpz_abs
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_abs
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_abs)
 __GMP_DECLSPEC void mpz_abs __GMP_PROTO ((mpz_ptr, mpz_srcptr));
 #endif
 
@@ -749,17 +766,17 @@ __GMP_DECLSPEC int mpz_fits_slong_p __GMP_PROTO ((mpz_srcptr)) __GMP_NOTHROW __G
 __GMP_DECLSPEC int mpz_fits_sshort_p __GMP_PROTO ((mpz_srcptr)) __GMP_NOTHROW __GMP_ATTRIBUTE_PURE;
 
 #define mpz_fits_uint_p __gmpz_fits_uint_p
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_fits_uint_p
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_fits_uint_p)
 __GMP_DECLSPEC int mpz_fits_uint_p __GMP_PROTO ((mpz_srcptr)) __GMP_NOTHROW __GMP_ATTRIBUTE_PURE;
 #endif
 
 #define mpz_fits_ulong_p __gmpz_fits_ulong_p
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_fits_ulong_p
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_fits_ulong_p)
 __GMP_DECLSPEC int mpz_fits_ulong_p __GMP_PROTO ((mpz_srcptr)) __GMP_NOTHROW __GMP_ATTRIBUTE_PURE;
 #endif
 
 #define mpz_fits_ushort_p __gmpz_fits_ushort_p
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_fits_ushort_p
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_fits_ushort_p)
 __GMP_DECLSPEC int mpz_fits_ushort_p __GMP_PROTO ((mpz_srcptr)) __GMP_NOTHROW __GMP_ATTRIBUTE_PURE;
 #endif
 
@@ -785,12 +802,12 @@ __GMP_DECLSPEC /* signed */ long int mpz_get_si __GMP_PROTO ((mpz_srcptr)) __GMP
 __GMP_DECLSPEC char *mpz_get_str __GMP_PROTO ((char *, int, mpz_srcptr));
 
 #define mpz_get_ui __gmpz_get_ui
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_get_ui
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_get_ui)
 __GMP_DECLSPEC unsigned long int mpz_get_ui __GMP_PROTO ((mpz_srcptr)) __GMP_NOTHROW __GMP_ATTRIBUTE_PURE;
 #endif
 
 #define mpz_getlimbn __gmpz_getlimbn
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_getlimbn
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_getlimbn)
 __GMP_DECLSPEC mp_limb_t mpz_getlimbn __GMP_PROTO ((mpz_srcptr, mp_size_t)) __GMP_NOTHROW __GMP_ATTRIBUTE_PURE;
 #endif
 
@@ -887,7 +904,7 @@ __GMP_DECLSPEC void mpz_mul_si __GMP_PROTO ((mpz_ptr, mpz_srcptr, long int));
 __GMP_DECLSPEC void mpz_mul_ui __GMP_PROTO ((mpz_ptr, mpz_srcptr, unsigned long int));
 
 #define mpz_neg __gmpz_neg
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_neg
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_neg)
 __GMP_DECLSPEC void mpz_neg __GMP_PROTO ((mpz_ptr, mpz_srcptr));
 #endif
 
@@ -908,12 +925,12 @@ __GMP_DECLSPEC size_t mpz_out_str __GMP_PROTO ((FILE *, int, mpz_srcptr));
 __GMP_DECLSPEC int mpz_perfect_power_p __GMP_PROTO ((mpz_srcptr)) __GMP_ATTRIBUTE_PURE;
 
 #define mpz_perfect_square_p __gmpz_perfect_square_p
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_perfect_square_p
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_perfect_square_p)
 __GMP_DECLSPEC int mpz_perfect_square_p __GMP_PROTO ((mpz_srcptr)) __GMP_ATTRIBUTE_PURE;
 #endif
 
 #define mpz_popcount __gmpz_popcount
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_popcount
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_popcount)
 __GMP_DECLSPEC unsigned long int mpz_popcount __GMP_PROTO ((mpz_srcptr)) __GMP_NOTHROW __GMP_ATTRIBUTE_PURE;
 #endif
 
@@ -963,7 +980,7 @@ __GMP_DECLSPEC void mpz_set_d __GMP_PROTO ((mpz_ptr, double));
 __GMP_DECLSPEC void mpz_set_f __GMP_PROTO ((mpz_ptr, mpf_srcptr));
 
 #define mpz_set_q __gmpz_set_q
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_set_q
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_set_q)
 __GMP_DECLSPEC void mpz_set_q __GMP_PROTO ((mpz_ptr, mpq_srcptr));
 #endif
 
@@ -980,7 +997,7 @@ __GMP_DECLSPEC void mpz_set_ui __GMP_PROTO ((mpz_ptr, unsigned long int));
 __GMP_DECLSPEC void mpz_setbit __GMP_PROTO ((mpz_ptr, unsigned long int));
 
 #define mpz_size __gmpz_size
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpz_size
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpz_size)
 __GMP_DECLSPEC size_t mpz_size __GMP_PROTO ((mpz_srcptr)) __GMP_NOTHROW __GMP_ATTRIBUTE_PURE;
 #endif
 
@@ -1058,7 +1075,7 @@ __GMP_DECLSPEC void mpz_xor __GMP_PROTO ((mpz_ptr, mpz_srcptr, mpz_srcptr));
 /**************** Rational (i.e. Q) routines.  ****************/
 
 #define mpq_abs __gmpq_abs
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpq_abs
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpq_abs)
 __GMP_DECLSPEC void mpq_abs __GMP_PROTO ((mpq_ptr, mpq_srcptr));
 #endif
 
@@ -1119,7 +1136,7 @@ __GMP_DECLSPEC void mpq_mul __GMP_PROTO ((mpq_ptr, mpq_srcptr, mpq_srcptr));
 __GMP_DECLSPEC void mpq_mul_2exp __GMP_PROTO ((mpq_ptr, mpq_srcptr, unsigned long));
 
 #define mpq_neg __gmpq_neg
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpq_neg
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpq_neg)
 __GMP_DECLSPEC void mpq_neg __GMP_PROTO ((mpq_ptr, mpq_srcptr));
 #endif
 
@@ -1368,12 +1385,12 @@ __GMP_DECLSPEC void mpf_urandomb __GMP_PROTO ((mpf_t, gmp_randstate_t, unsigned
 /* This is ugly, but we need to make user calls reach the prefixed function. */
 
 #define mpn_add __MPN(add)
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpn_add
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpn_add)
 __GMP_DECLSPEC mp_limb_t mpn_add __GMP_PROTO ((mp_ptr, mp_srcptr, mp_size_t, mp_srcptr,mp_size_t));
 #endif
 
 #define mpn_add_1 __MPN(add_1)
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpn_add_1
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpn_add_1)
 __GMP_DECLSPEC mp_limb_t mpn_add_1 __GMP_PROTO ((mp_ptr, mp_srcptr, mp_size_t, mp_limb_t)) __GMP_NOTHROW;
 #endif
 
@@ -1387,18 +1404,18 @@ __GMP_DECLSPEC mp_limb_t mpn_addmul_1 __GMP_PROTO ((mp_ptr, mp_srcptr, mp_size_t
 __GMP_DECLSPEC mp_limb_t mpn_bdivmod __GMP_PROTO ((mp_ptr, mp_ptr, mp_size_t, mp_srcptr, mp_size_t, unsigned long int));
 
 #define mpn_cmp __MPN(cmp)
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpn_cmp
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpn_cmp)
 __GMP_DECLSPEC int mpn_cmp __GMP_PROTO ((mp_srcptr, mp_srcptr, mp_size_t)) __GMP_NOTHROW __GMP_ATTRIBUTE_PURE;
 #endif
 
 #define mpn_divexact_by3(dst,src,size) \
-  mpn_divexact_by3c (dst, src, size, (mp_limb_t) 0)
+  mpn_divexact_by3c (dst, src, size, __GMP_CAST (mp_limb_t, 0))
 
 #define mpn_divexact_by3c __MPN(divexact_by3c)
 __GMP_DECLSPEC mp_limb_t mpn_divexact_by3c __GMP_PROTO ((mp_ptr, mp_srcptr, mp_size_t, mp_limb_t));
 
 #define mpn_divmod_1(qp,np,nsize,dlimb) \
-  mpn_divrem_1 (qp, (mp_size_t) 0, np, nsize, dlimb)
+  mpn_divrem_1 (qp, __GMP_CAST (mp_size_t, 0), np, nsize, dlimb)
 
 #define mpn_divrem __MPN(divrem)
 __GMP_DECLSPEC mp_limb_t mpn_divrem __GMP_PROTO ((mp_ptr, mp_size_t, mp_ptr, mp_size_t, mp_srcptr, mp_size_t));
@@ -1474,12 +1491,12 @@ __GMP_DECLSPEC mp_size_t mpn_set_str __GMP_PROTO ((mp_ptr, __gmp_const unsigned
 __GMP_DECLSPEC mp_size_t mpn_sqrtrem __GMP_PROTO ((mp_ptr, mp_ptr, mp_srcptr, mp_size_t));
 
 #define mpn_sub __MPN(sub)
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpn_sub
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpn_sub)
 __GMP_DECLSPEC mp_limb_t mpn_sub __GMP_PROTO ((mp_ptr, mp_srcptr, mp_size_t, mp_srcptr,mp_size_t));
 #endif
 
 #define mpn_sub_1 __MPN(sub_1)
-#if __GMP_INLINE_PROTOTYPES || __GMP_FORCE_mpn_sub_1
+#if __GMP_INLINE_PROTOTYPES || defined (__GMP_FORCE_mpn_sub_1)
 __GMP_DECLSPEC mp_limb_t mpn_sub_1 __GMP_PROTO ((mp_ptr, mp_srcptr, mp_size_t, mp_limb_t)) __GMP_NOTHROW;
 #endif
 
@@ -1509,7 +1526,7 @@ __GMP_DECLSPEC void mpn_tdiv_qr __GMP_PROTO ((mp_ptr, mp_ptr, mp_size_t, mp_srcp
    is wanted for the inline than for the library version, then
    __GMP_FORCE_foo arranges the inline to be suppressed, eg. mpz_abs.  */
 
-#if defined (__GMP_EXTERN_INLINE) && ! __GMP_FORCE_mpz_abs
+#if defined (__GMP_EXTERN_INLINE) && ! defined (__GMP_FORCE_mpz_abs)
 __GMP_EXTERN_INLINE void
 mpz_abs (mpz_ptr __gmp_w, mpz_srcptr __gmp_u)
 {
@@ -1532,8 +1549,8 @@ mpz_abs (mpz_ptr __gmp_w, mpz_srcptr __gmp_u)
 	  || (__gmp_n == 2 && __gmp_p[1] <= ((mp_limb_t) maxval >> GMP_NUMB_BITS)));
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpz_fits_uint_p
-#if ! __GMP_FORCE_mpz_fits_uint_p
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpz_fits_uint_p)
+#if ! defined (__GMP_FORCE_mpz_fits_uint_p)
 __GMP_EXTERN_INLINE
 #endif
 int
@@ -1543,8 +1560,8 @@ mpz_fits_uint_p (mpz_srcptr __gmp_z) __GMP_NOTHROW
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpz_fits_ulong_p
-#if ! __GMP_FORCE_mpz_fits_ulong_p
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpz_fits_ulong_p)
+#if ! defined (__GMP_FORCE_mpz_fits_ulong_p)
 __GMP_EXTERN_INLINE
 #endif
 int
@@ -1554,8 +1571,8 @@ mpz_fits_ulong_p (mpz_srcptr __gmp_z) __GMP_NOTHROW
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpz_fits_ushort_p
-#if ! __GMP_FORCE_mpz_fits_ushort_p
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpz_fits_ushort_p)
+#if ! defined (__GMP_FORCE_mpz_fits_ushort_p)
 __GMP_EXTERN_INLINE
 #endif
 int
@@ -1565,8 +1582,8 @@ mpz_fits_ushort_p (mpz_srcptr __gmp_z) __GMP_NOTHROW
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpz_get_ui
-#if ! __GMP_FORCE_mpz_get_ui
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpz_get_ui)
+#if ! defined (__GMP_FORCE_mpz_get_ui)
 __GMP_EXTERN_INLINE
 #endif
 unsigned long
@@ -1590,8 +1607,8 @@ mpz_get_ui (mpz_srcptr __gmp_z) __GMP_NOTHROW
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpz_getlimbn
-#if ! __GMP_FORCE_mpz_getlimbn
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpz_getlimbn)
+#if ! defined (__GMP_FORCE_mpz_getlimbn)
 __GMP_EXTERN_INLINE
 #endif
 mp_limb_t
@@ -1604,7 +1621,7 @@ mpz_getlimbn (mpz_srcptr __gmp_z, mp_size_t __gmp_n) __GMP_NOTHROW
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) && ! __GMP_FORCE_mpz_neg
+#if defined (__GMP_EXTERN_INLINE) && ! defined (__GMP_FORCE_mpz_neg)
 __GMP_EXTERN_INLINE void
 mpz_neg (mpz_ptr __gmp_w, mpz_srcptr __gmp_u)
 {
@@ -1614,8 +1631,8 @@ mpz_neg (mpz_ptr __gmp_w, mpz_srcptr __gmp_u)
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpz_perfect_square_p
-#if ! __GMP_FORCE_mpz_perfect_square_p
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpz_perfect_square_p)
+#if ! defined (__GMP_FORCE_mpz_perfect_square_p)
 __GMP_EXTERN_INLINE
 #endif
 int
@@ -1629,8 +1646,8 @@ mpz_perfect_square_p (mpz_srcptr __gmp_a)
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpz_popcount
-#if ! __GMP_FORCE_mpz_popcount
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpz_popcount)
+#if ! defined (__GMP_FORCE_mpz_popcount)
 __GMP_EXTERN_INLINE
 #endif
 unsigned long
@@ -1645,8 +1662,8 @@ mpz_popcount (mpz_srcptr __gmp_u) __GMP_NOTHROW
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpz_set_q
-#if ! __GMP_FORCE_mpz_set_q
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpz_set_q)
+#if ! defined (__GMP_FORCE_mpz_set_q)
 __GMP_EXTERN_INLINE
 #endif
 void
@@ -1656,8 +1673,8 @@ mpz_set_q (mpz_ptr __gmp_w, mpq_srcptr __gmp_u)
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpz_size
-#if ! __GMP_FORCE_mpz_size
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpz_size)
+#if ! defined (__GMP_FORCE_mpz_size)
 __GMP_EXTERN_INLINE
 #endif
 size_t
@@ -1670,7 +1687,7 @@ mpz_size (mpz_srcptr __gmp_z) __GMP_NOTHROW
 
 /**************** mpq inlines ****************/
 
-#if defined (__GMP_EXTERN_INLINE) && ! __GMP_FORCE_mpq_abs
+#if defined (__GMP_EXTERN_INLINE) && ! defined (__GMP_FORCE_mpq_abs)
 __GMP_EXTERN_INLINE void
 mpq_abs (mpq_ptr __gmp_w, mpq_srcptr __gmp_u)
 {
@@ -1680,7 +1697,7 @@ mpq_abs (mpq_ptr __gmp_w, mpq_srcptr __gmp_u)
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) && ! __GMP_FORCE_mpq_neg
+#if defined (__GMP_EXTERN_INLINE) && ! defined (__GMP_FORCE_mpq_neg)
 __GMP_EXTERN_INLINE void
 mpq_neg (mpq_ptr __gmp_w, mpq_srcptr __gmp_u)
 {
@@ -1959,8 +1976,8 @@ mpq_neg (mpq_ptr __gmp_w, mpq_srcptr __gmp_u)
 #endif
 
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpn_add
-#if ! __GMP_FORCE_mpn_add
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpn_add)
+#if ! defined (__GMP_FORCE_mpn_add)
 __GMP_EXTERN_INLINE
 #endif
 mp_limb_t
@@ -1972,8 +1989,8 @@ mpn_add (mp_ptr __gmp_wp, mp_srcptr __gmp_xp, mp_size_t __gmp_xsize, mp_srcptr _
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpn_add_1
-#if ! __GMP_FORCE_mpn_add_1
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpn_add_1)
+#if ! defined (__GMP_FORCE_mpn_add_1)
 __GMP_EXTERN_INLINE
 #endif
 mp_limb_t
@@ -1985,8 +2002,8 @@ mpn_add_1 (mp_ptr __gmp_dst, mp_srcptr __gmp_src, mp_size_t __gmp_size, mp_limb_
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpn_cmp
-#if ! __GMP_FORCE_mpn_cmp
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpn_cmp)
+#if ! defined (__GMP_FORCE_mpn_cmp)
 __GMP_EXTERN_INLINE
 #endif
 int
@@ -1998,8 +2015,8 @@ mpn_cmp (mp_srcptr __gmp_xp, mp_srcptr __gmp_yp, mp_size_t __gmp_size) __GMP_NOT
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpn_sub
-#if ! __GMP_FORCE_mpn_sub
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpn_sub)
+#if ! defined (__GMP_FORCE_mpn_sub)
 __GMP_EXTERN_INLINE
 #endif
 mp_limb_t
@@ -2011,8 +2028,8 @@ mpn_sub (mp_ptr __gmp_wp, mp_srcptr __gmp_xp, mp_size_t __gmp_xsize, mp_srcptr _
 }
 #endif
 
-#if defined (__GMP_EXTERN_INLINE) || __GMP_FORCE_mpn_sub_1
-#if ! __GMP_FORCE_mpn_sub_1
+#if defined (__GMP_EXTERN_INLINE) || defined (__GMP_FORCE_mpn_sub_1)
+#if ! defined (__GMP_FORCE_mpn_sub_1)
 __GMP_EXTERN_INLINE
 #endif
 mp_limb_t
@@ -2042,14 +2059,14 @@ mpn_sub_1 (mp_ptr __gmp_dst, mp_srcptr __gmp_src, mp_size_t __gmp_size, mp_limb_
 #define mpz_cmp_si(Z,SI) \
   (__builtin_constant_p (SI) && (SI) == 0 ? mpz_sgn (Z)			\
    : __builtin_constant_p (SI) && (SI) > 0				\
-    ? _mpz_cmp_ui (Z, (unsigned long int) SI)				\
+    ? _mpz_cmp_ui (Z, __GMP_CAST (unsigned long int, SI))               \
    : _mpz_cmp_si (Z,SI))
 #define mpq_cmp_ui(Q,NUI,DUI) \
   (__builtin_constant_p (NUI) && (NUI) == 0				\
    ? mpq_sgn (Q) : _mpq_cmp_ui (Q,NUI,DUI))
 #define mpq_cmp_si(q,n,d)                       \
   (__builtin_constant_p ((n) >= 0) && (n) >= 0  \
-   ? mpq_cmp_ui (q, (unsigned long) (n), d)     \
+   ? mpq_cmp_ui (q, __GMP_CAST (unsigned long, n), d) \
    : _mpq_cmp_si (q, n, d))
 #else
 #define mpz_cmp_ui(Z,UI) _mpz_cmp_ui (Z,UI)
@@ -2062,7 +2079,7 @@ mpn_sub_1 (mp_ptr __gmp_dst, mp_srcptr __gmp_src, mp_size_t __gmp_size, mp_limb_
 /* Using "&" rather than "&&" means these can come out branch-free.  Every
    mpz_t has at least one limb allocated, so fetching the low limb is always
    allowed.  */
-#define mpz_odd_p(z)   ((int) ((z)->_mp_size != 0) & (int) (z)->_mp_d[0])
+#define mpz_odd_p(z)   (((z)->_mp_size != 0) & __GMP_CAST (int, (z)->_mp_d[0]))
 #define mpz_even_p(z)  (! mpz_odd_p (z))
 
 
@@ -2088,9 +2105,9 @@ __GMP_DECLSPEC_XX std::istream& operator>> (std::istream &, mpf_ptr);
 #define mpz_mmod	mpz_fdiv_r
 #define mpz_mdiv_ui	mpz_fdiv_q_ui
 #define mpz_mdivmod_ui(q,r,n,d) \
-  ((r == 0) ? mpz_fdiv_q_ui (q,n,d) : mpz_fdiv_qr_ui (q,r,n,d))
+  (((r) == 0) ? mpz_fdiv_q_ui (q,n,d) : mpz_fdiv_qr_ui (q,r,n,d))
 #define mpz_mmod_ui(r,n,d) \
-  ((r == 0) ? mpz_fdiv_ui (n,d) : mpz_fdiv_r_ui (r,n,d))
+  (((r) == 0) ? mpz_fdiv_ui (n,d) : mpz_fdiv_r_ui (r,n,d))
 
 /* Useful synonyms, but not quite compatible with GMP 1.  */
 #define mpz_div		mpz_fdiv_q
@@ -2116,7 +2133,7 @@ enum
 /* Major version number is the value of __GNU_MP__ too, above and in mp.h. */
 #define __GNU_MP_VERSION 4
 #define __GNU_MP_VERSION_MINOR 1
-#define __GNU_MP_VERSION_PATCHLEVEL 2
+#define __GNU_MP_VERSION_PATCHLEVEL 4
 
 #define __GMP_H__
 #endif /* __GMP_H__ */
